//
//  SpringModel.swift
//  Muelle Amortiguado
//
//  Created by Santiago Pavón on 1/10/15.
//  Copyright © 2015 UPM. All rights reserved.
//

import Foundation


class SpringModel {
    
    // Masa del cuerpo
    var mass = 10.0 {
        didSet {
            updateCtes()
        }
    }
    
    // Constante de Muelle
    var k = 1000.0 {
        didSet {
            updateCtes()
        }
    }
    
    // Constante rozamiento
    var λ = 20.0 {
        didSet {
            updateCtes()
        }
    }

    init() {
      updateCtes()
    }
    
    // Posicion inicial de la masa
    private let x0 = 2.0
    
    // Velocidad inicial de la masa
    private let v0 = 0.0
    
    // Ctes intermedias
    private var ω0 = 1.0
    private var γ = 1.0
    private var ω = 1.0
    private var A = 1.0
    private var φ = 1.0

    // Actualizar las ctest intermedias.
    private func updateCtes() {
         ω0 = sqrt(k / mass)
         γ = λ / mass / 2
         ω = sqrt(ω0*ω0 - γ*γ)
         A = sqrt(x0*x0 + pow((v0+γ*x0)/ω,2))
         φ = atan(x0*ω/(v0+γ*x0))
    }
    
    // Dame la posicion de la masa en el instante t
    func posAtTime(_ t:Double) -> Double {
        return A*exp(-γ*t) * sin(ω*t + φ)
    }
    
    // Dame la velocidad de la masa en el instante t
    func speedAtTime(_ t:Double) -> Double {
        
        let ae = A*exp(-γ*t)
        let ωtφ = ω*t+φ
        return -γ*ae*sin(ωtφ) + ae*ω*cos(ωtφ)
    }
    
}
